# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager osdk-specific functionality.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This subpackage includes necessary modules for manipulating the operator-sdk
installation itself.
"""



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager update module.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains the code to update the installed Operator SDK binaries.
"""

import hashlib
import logging
import os
import requests
from lastversion.lastversion import latest as lastversion
from tempfile import mkstemp
from pathlib import Path
from typing import List

from osdk_manager.util import get_logger, GpgTrust

_called_from_test = False

# TODO: Go full classful


class OsdkFileData(object):
    """A basic class to build paths for osdk updates."""

    def __init__(self, version: str = None, arch: str = 'linux_amd64',
                 path: Path = os.path.expanduser('~/.local/bin'),
                 verify: bool = True) -> None:
        """Initialize a simple tracker for OSDK-related paths."""
        osdk_download_url = 'https://github.com/operator-framework/operator-sdk/releases/download'  # noqa: E501
        download_base_url = f'{osdk_download_url}/v{version}'
        osdk_downloads = ['operator-sdk', 'ansible-operator', 'helm-operator']

        self.logger = get_logger()
        self.logger.debug(self.__dict__)

        self.hashes = requests.get(
            f'{download_base_url}/checksums.txt'
        ).content
        self.hash_signature = requests.get(
            f'{download_base_url}/checksums.txt.asc'
        ).content if verify else None

        self.downloads = {}
        for download in osdk_downloads:
            filename = f'{download}_{arch}'
            file_data = {
                'filename': filename,
                'url': f'{download_base_url}/{filename}',
                'dst': f'{path}/{download}'
            }
            for sha256 in self.hashes.decode().split('\n'):
                if sha256.endswith(f' {file_data["filename"]}'):
                    file_data.update({"hash": sha256.split()[0]})
                    break
            self.logger.debug({download: file_data})
            self.downloads[download] = file_data

    def files_not_matching(self) -> List[str]:
        """Return all of the download names that don't pass checksum."""
        not_matching = []
        for download in self.downloads:
            filename = self.downloads[download]['dst']
            expected_hash = self.downloads[download]['hash']
            self.logger.debug(
                f'Checking {filename} for expected hash {expected_hash}'
            )
            if os.path.exists(filename):
                existing_hash = hashlib.sha256()
                with open(filename, 'rb') as f:
                    existing_hash.update(f.read())
                if existing_hash.hexdigest() != expected_hash:
                    self.logger.info((f'Hash for {filename} does not match'
                                      f' expected hash, {expected_hash}.'))
                    not_matching.append(download)
                else:
                    self.logger.info(f'Hash for {filename} appears current.')
            else:
                self.logger.info(f'{filename} not present.')
                not_matching.append(download)
        self.logger.debug(f'not_matching: {not_matching}')
        return not_matching


def osdk_update(path: str = os.path.expanduser('~/.local/bin'),
                version: str = 'latest', verify: bool = True) -> str:
    """Update the operator-sdk binaries."""
    logger = get_logger()
    for arg in [path, version, verify]:
        logger.debug(type(arg))
        logger.debug(arg)

    logger.debug(f'Creating {path}')
    os.makedirs(path, exist_ok=True)

    if version == 'latest':
        logger.debug('Determining latest version of the operator-sdk')
        version = lastversion('operator-framework/operator-sdk')
        # lastversion sets handlers on the root logger because it's mean.
        if not _called_from_test:  # pragma: no cover
            root_logger = logging.getLogger()
            root_logger.handlers.clear()

    if len(str(version)) < 1:  # pragma: no cover
        raise RuntimeError(('Unable to determine latest version. '
                            'Consider specifying it with the --version '
                            'option at the command line.'))

    logger.info(f'Identified desired installation version as {version}')
    osdk_file_data = OsdkFileData(version=version, path=path)

    if verify:
        gpg = GpgTrust()
        gpg.trust('3B2F1481D146238080B346BB052996E2A20B5C7E')
        hashes_fd, hashes_path = mkstemp()
        with os.fdopen(hashes_fd, 'wb') as f:
            f.write(osdk_file_data.hashes)
        try:
            gpg.verify(hashes_path, osdk_file_data.hash_signature)
        finally:
            os.remove(hashes_path)
    else:
        logger.warning('Not validating signatures as requested.')

    for download in osdk_file_data.files_not_matching():
        data = osdk_file_data.downloads[download]
        binary = requests.get(data["url"]).content
        logger.info(f'Writing {data["dst"]}.')
        with open(data["dst"], 'wb') as f:
            f.write(binary)

        mode = os.stat(data["dst"]).st_mode
        mode_ex = mode | 0o111
        if mode != mode_ex:
            logger.info(f'Making {data["dst"]} executable.')
            os.chmod(data["dst"], mode_ex & 0o7777)

    return str(version)



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager command line interface.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains the main CLI interfaces, based on Click.
"""

import click
import sys

from osdk_manager.util import get_logger
from .util import verbose_opt

logger = get_logger()


@click.group()
@verbose_opt
@click.version_option()
def cli(verbose):
    """Operator SDK Manager.

    Manage the operator-sdk binary and associated dependencies.
    """
    logger = get_logger(verbose)
    logger.debug(sys.argv)
    logger.debug(f'verbose: {verbose}')


import osdk_manager.cli.osdk  # noqa E402
import osdk_manager.cli.opm  # noqa E402



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager package.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.
"""

import platform

if platform.system() != "Linux":  # pragma: no cover
    raise EnvironmentError("osdk_manager is designed only for Linux.")



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager exceptions.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains the custom exceptions utilized for the osdk_manager.
"""


class ContainerRuntimeException(Exception):
    """Unable to identify a container runtime in your current environment."""

    pass


class ShellRuntimeException(RuntimeError):
    """Shell command returned non-zero return code.

    Attributes:
        code -- the return code from the shell command
    """

    def __init__(self, code: int = None):
        """Save the code with the exception."""
        self.code = code



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager module.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.
"""

from osdk_manager.cli import cli

cli()



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager opm-specific functionality.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This subpackage includes necessary modules for updating the installation of the
Operator Package Manager, opm.
"""

import hashlib
import logging
import os
import requests
from lastversion.lastversion import latest as lastversion

from osdk_manager.util import get_logger

_called_from_test = False


class OpmPaths(object):
    """A basic class to build paths for opm updates."""

    def __init__(self, version: str = None, arch: str = 'linux-amd64',
                 directory: str = os.path.expanduser('~/.operator-sdk'),
                 path: str = os.path.expanduser('~/.local/bin')) -> None:
        """Initialize a simple tracker for OPM-related paths."""
        download_base_url = (f'https://github.com/operator-framework/'
                             f'operator-registry/releases/download/v{version}')
        self.filename = f'{arch}-opm'
        self.download_url = f'{download_base_url}/{self.filename}'
        self.src = f'{directory}/{self.filename}-{version}'
        self.dst = f'{path}/opm'
        logger = get_logger()
        logger.debug(self.__dict__)


def opm_version(directory: str = os.path.expanduser('~/.operator-sdk'),
                path: str = os.path.expanduser('~/.local/bin')) -> str:
    """Return the version of the installed opm binary."""
    logger = get_logger()
    for arg in [directory, path]:
        logger.debug(type(arg))
        logger.debug(arg)

    link_path = os.path.join(path, 'opm')
    if os.path.islink(link_path):
        target = os.readlink(link_path)
        binary = os.path.basename(target)
        assumed_version = binary.split('-')[-1]
    else:
        logger.info('Unable to identify opm symlink.')
        return ''
    paths = OpmPaths(version=assumed_version, directory=directory, path=path)
    if not os.readlink(paths.dst) == paths.src:
        logger.info(f'opm not symlinked into {path}.')
        return ''
    if not os.path.isfile(paths.src):
        logger.info(f'{paths.dst} is a dangling link to {paths.src}')
        return ''

    return assumed_version


def opm_update(directory: str = os.path.expanduser('~/.operator-sdk'),
               path: str = os.path.expanduser('~/.local/bin'),
               version: str = 'latest') -> str:
    """Update the opm binary."""
    logger = get_logger()
    for arg in [directory, path, version]:
        logger.debug(type(arg))
        logger.debug(arg)

    logger.debug(f'Creating {directory}')
    os.makedirs(directory, exist_ok=True)

    logger.debug(f'Creating {path}')
    os.makedirs(path, exist_ok=True)

    if version == 'latest':
        logger.debug('Determining latest version of opm')
        version = lastversion('operator-framework/operator-registry')
        # lastversion sets handlers on the root logger because it's mean.
        if not _called_from_test:  # pragma: no cover
            root_logger = logging.getLogger()
            root_logger.handlers.clear()

    if len(str(version)) < 1:  # pragma: no cover
        raise RuntimeError(('Unable to determine latest version. '
                            'Consider specifying it with the --version '
                            'option at the command line.'))

    logger.info(f'Identified desired installation version as {version}')
    installed_version = opm_version(directory=directory, path=path)

    if version == installed_version:
        logger.info(f'{version} is already installed.')
        return version

    paths = OpmPaths(version=version, directory=directory, path=path)
    if os.path.isfile(paths.src):
        logger.debug(f'Already downloaded: {paths.filename}')
    else:
        logger.debug(f'Requesting {paths.download_url}')
        binary = requests.get(paths.download_url).content
        logger.debug(f'Saving {paths.download_url}')
        with open(paths.src, 'wb') as f:
            f.write(binary)

        binary_sha256 = hashlib.sha256(binary).hexdigest()
        logger.debug(f'Saved {paths.dst} with SHA 256 {binary_sha256}')

    src_mode = os.stat(paths.src).st_mode
    src_mode_ex = src_mode | 0o111
    if src_mode != src_mode_ex:
        logger.info(f'Making {paths.src} executable.')
        os.chmod(paths.src, src_mode_ex & 0o7777)

    if os.path.islink(paths.dst):
        if os.readlink(paths.dst) == paths.src:
            logger.debug(f'Already linked {paths.src} to {paths.dst}')
        else:
            logger.info(f'Symlinking {paths.src} to {paths.dst}')
            os.remove(paths.dst)
            os.symlink(paths.src, paths.dst)
    else:
        logger.info(f'Symlinking {paths.src} to {paths.dst}')
        os.symlink(paths.src, paths.dst)

    return str(version)



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager Operator scaffolding functionality.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This module includes the class that scaffolds, builds, validates, and pushes
Operator-related images with the necessary context.
"""

import os
import yaml
from typing import TypeVar, List

from osdk_manager.util import determine_runtime, get_logger, shell


T = TypeVar("Operator")


class Operator(object):
    """A basic abstraction around manipulating operator repos and images.

    Executes various operator-sdk and opm commands as necessary to manage your
    operator project.
    """

    logger = get_logger()

    def __init__(self, directory: str = None, image: str = None,
                 version: str = None, channels: List[str] = [],
                 kinds: List[str] = [], default_sample: str = None,
                 domain: str = None, group: str = None,
                 api_version: str = None, initialized: bool = False,
                 runtime: str = None) -> None:
        """Initialize an Operator with the necessary variables."""
        self.directory = directory
        os.chdir(self.directory)
        if '/' in image:
            self.image = image
        else:
            self.image = "localhost/{}".format(image)
        self.version = version
        self.tag = version
        self.channels = channels
        self.kinds = kinds
        self.default_sample = default_sample
        self.domain = domain
        self.group = group
        self.api_version = api_version
        self.initialized = initialized
        if runtime is not None:
            self.runtime = runtime
        else:
            self.runtime = determine_runtime()

    def __repr__(self) -> str:
        """Represent the state of this object as a string."""
        return ("Operator(directory={}, image={}, version={}, tag={},"
                " channels={}, kinds={}, default_sample={}, domain={},"
                " group={}, api_version={}, initialized={},"
                " runtime={})").format(
            self.directory,
            self.image,
            self.version,
            self.tag,
            self.channels,
            self.kinds,
            self.default_sample,
            self.domain,
            self.group,
            self.api_version,
            self.initialized,
            self.runtime
        )

    @classmethod
    def load(cls, directory: str = '.', filename: str = "operate.yml",
             runtime: str = None) -> T:
        """Alternate constructor to load settings from a yaml file."""
        with open(os.path.join(directory, filename)) as f:
            settings = yaml.safe_load(f)
        cls.logger.debug("Recovered settings:")
        cls.logger.debug(settings)

        return cls(directory=os.path.realpath(directory),
                   image=settings.get("image"),
                   version=settings.get("version"),
                   channels=settings.get("channels"),
                   kinds=settings.get("kinds"),
                   default_sample=settings.get("default-sample"),
                   domain=settings.get("domain"),
                   group=settings.get("group"),
                   api_version=settings.get("api-version"),
                   runtime=runtime)

    def initialize_ansible_operator(self) -> None:
        """Initialize an Ansible Operator SDK operator.

        Also creates APIs represented by the Kinds specified.
        """
        if self.initialized:
            return

        [line for line in shell(
            "operator-sdk init --plugins=ansible --domain={}".format(
                self.domain
            )
        )]
        [[line for line in shell(
            "operator-sdk create api --group={} --version={} --kind={}".format(
                self.group, self.api_version, kind
            )
        )] for kind in self.kinds]

        self.initialized = True

    def _set_vars(self) -> None:
        """Export appropriate variables into the environment for the SDK."""
        os.environ["IMG"] = ':'.join([self.image, self.tag])
        os.environ["BUNDLE_IMG"] = ':'.join([
            self.image + "-operator", self.tag
        ])
        if len(self.channels) > 0:
            os.environ["BUNDLE_CHANNELS"] = ','.join(self.channels)
            os.environ["BUNDLE_DEFAULT_CHANNEL"] = self.channels[0]

    def set_tag(self, tag: str = None) -> None:
        """Set tag to the desired tag, or our version."""
        if tag:
            self.tag = tag
        else:
            self.tag = self.version

    def login(self, username: str = None, password: str = None) -> None:
        """Log in to the repository for the current image.

        Will raise an exception if unable to log in.
        """
        [line for line in shell("{} login -u {} -p {} {}".format(
            self.runtime, username, password, self.image.split('/', 2)[0]
        ))]

    def build(self) -> str:
        """Build an operator image using the saved values.

        Returns the image name and tag as a string.
        """
        self._set_vars()
        [line for line in shell("{} build . -t {}".format(self.runtime,
                                                          os.getenv("IMG")))]
        return os.getenv("IMG")

    def get_images(self) -> List[str]:
        """Return a list of all images related to this operator."""
        ret = []
        for line in shell(self.runtime +
                          " images --format='{{.Repository}}:{{.Tag}}'"):
            if line.startswith(self.image):
                ret.append(line.strip())
        return ret

    def remove_images(self) -> None:
        """Remove all identified images that belong to this operator."""
        for image in self.get_images():
            [line for line in shell(self.runtime + " rmi -f " + image)]



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager command line OSDK commands.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains the CLI subcommands directly related to the Operator SDK
binaries.
"""

import click
import os

from osdk_manager.cli import cli
from osdk_manager.cli.util import verbose_opt
from osdk_manager.util import get_logger


@cli.group()
@verbose_opt
def osdk(verbose):
    """Manage the operator-sdk installation."""
    logger = get_logger(verbose)
    logger.debug(f'verbose: {verbose}')


@osdk.command()
@verbose_opt
@click.option('-p', '--path', default=os.path.expanduser('~/.local/bin'),
              help='The directory in your $PATH to install operator-sdk into')
@click.option('-V', '--version', default='latest',
              help='The version of the Operator SDK to install')
@click.option('-n', '--no-verify', is_flag=True,
              help="Don't verify GPG signatures")
def update(verbose, path, version, no_verify):
    """Update the operator-sdk binary, validating sums."""
    logger = get_logger(verbose)
    logger.debug(f'verbose: {verbose}')
    logger.debug(f'path: {path}')
    logger.debug(f'version: {version}')
    logger.debug(f'no_verify: {no_verify}')

    from osdk_manager.osdk.update import osdk_update
    version = osdk_update(path=path, version=version, verify=not no_verify)

    if path in os.getenv('PATH').split(':'):
        click.echo((f'operator-sdk version {version} is in your path as '
                    f'operator-sdk'))
    else:
        click.echo((f'operator-sdk version {version} is available at '
                    f'{path}/operator-sdk'))



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager Operator scaffolding functionality.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This subpackage includes the classes and modules required to scaffold, build,
validate, and push Operator-related repositories and images.
"""

from .operator import Operator  # noqa: F401



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager utilities.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains utilities utilized throughout the package and modules.
"""

import gnupg
import logging
import logging.handlers
import os
import shlex
import subprocess
from io import BytesIO
from pathlib import Path
from typing import List, Iterable

from osdk_manager.exceptions import (
    ContainerRuntimeException,
    ShellRuntimeException
)


def get_logger(verbosity: int = None):
    """Create a logger, or return an existing one with specified verbosity."""
    logger = logging.getLogger('osdk-manager')
    logger.setLevel(logging.DEBUG)

    if len(logger.handlers) == 0:
        _format = '{asctime} {name} [{levelname:^9s}]: {message}'
        formatter = logging.Formatter(_format, style='{')

        stderr = logging.StreamHandler()
        stderr.setFormatter(formatter)
        if verbosity is not None:
            stderr.setLevel(40 - (min(3, verbosity) * 10))
        else:
            stderr.setLevel(40)
        logger.addHandler(stderr)

        if os.path.exists('/dev/log'):
            syslog = logging.handlers.SysLogHandler(address='/dev/log')
            syslog.setFormatter(formatter)
            syslog.setLevel(logging.INFO)
            logger.addHandler(syslog)
    else:
        if verbosity is not None and verbosity != 0:
            stderr = logger.handlers[0]
            stderr.setLevel(40 - (min(3, verbosity) * 10))

    return logger


def _utf8ify(line_bytes: List[bytes] = None) -> str:
    """Decode line_bytes as utf-8 and strips excess whitespace."""
    return line_bytes.decode("utf-8").rstrip()


def shell(cmd: str = None, fail: bool = True) -> Iterable[str]:
    """Run a command in a subprocess, yielding lines of output from it.

    By default will cause a failure using the return code of the command. To
    change this behavior, pass fail=False.
    """
    logger = get_logger()
    logger.debug("Running: {}".format(cmd))
    proc = subprocess.Popen(shlex.split(cmd),
                            stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)

    for line in map(_utf8ify, iter(proc.stdout.readline, b'')):
        logger.debug("Line:    {}".format(line))
        yield line

    ret = proc.wait()
    if fail and ret != 0:
        logger.error("Command errored: {}".format(cmd))
        raise ShellRuntimeException(ret)
    elif ret != 0:
        logger.warning("Command returned {}: {}".format(ret, cmd))


def determine_runtime() -> str:  # pragma: no cover
    """Determine the container runtime installed on the system."""
    try:
        [line for line in shell("docker --help", fail=False)]
        return "docker"
    except FileNotFoundError:
        pass
    try:
        [line for line in shell("podman --help", fail=False)]
        return "podman"
    except FileNotFoundError:
        raise ContainerRuntimeException


class GpgTrust(object):
    """Handles GPG key trust and signature validation."""

    def __init__(self, key_server: str = 'keys.gnupg.net') -> None:
        """Initialize a GPG Trust database object."""
        self.logger = get_logger()
        self.gnupghome = os.path.expanduser('~/.gnupg')
        self.key_server = key_server

        self.logger.debug(f'Creating {self.gnupghome}')
        os.makedirs(self.gnupghome, mode=0o700, exist_ok=True)

        self.gpg = gnupg.GPG(gnupghome=self.gnupghome)

    def trust(self, key_id: str = None) -> bool:
        """Trust a GPG public key."""
        try:
            self.logger.debug(f'Importing key {key_id} from {self.key_server}')
            self.gpg.recv_keys(self.key_server, key_id)
        except FileNotFoundError:  # pragma: no cover
            self.logger.error('Unable to receive keys!')
            raise
        return True

    def verify(self, target: Path = None, signature: bytes = None) -> bool:
        """Validate a signature using the trust database."""
        self.logger.debug((f'Validating {target} signature'))

        signature = BytesIO(signature)

        if self.gpg.verify_file(signature, target):
            self.logger.debug(f'{target} verified.')
            return True
        else:
            raise RuntimeError(f'{target} failed verification.')



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager command line OPM commands.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains the CLI subcommands directly related to the Operator Package
Manager binaries.
"""

import click
import os

from osdk_manager.cli import cli
from osdk_manager.cli.util import verbose_opt
from osdk_manager.util import get_logger


@cli.group()
@verbose_opt
def opm(verbose):
    """Manage the opm installation."""
    logger = get_logger(verbose)
    logger.debug(f'verbose: {verbose}')


@opm.command()
@verbose_opt
@click.option('-d', '--directory',
              default=os.path.expanduser('~/.operator-sdk'),
              help='The directory into which to unpack opm')
@click.option('-p', '--path', default=os.path.expanduser('~/.local/bin'),
              help='The directory in your $PATH to symlink opm into')
@click.option('-V', '--version', default='latest',
              help='The version of the Operator Package Manager to install')
def update(verbose, directory, path, version):
    """Update the opm binary, validating sums."""
    logger = get_logger(verbose)
    logger.debug(f'verbose: {verbose}')
    logger.debug(f'directory: {directory}')
    logger.debug(f'path: {path}')
    logger.debug(f'version: {version}')

    from osdk_manager.opm.update import opm_update
    version = opm_update(directory=directory, path=path, version=version)

    if path in os.getenv('PATH').split(':'):
        click.echo(f'opm version {version} is in your path as opm')
    else:
        click.echo(f'opm version {version} is available at {path}/opm')


@opm.command()
@verbose_opt
@click.option('-d', '--directory',
              default=os.path.expanduser('~/.operator-sdk'),
              help='The directory in which to look for the opm')
@click.option('-p', '--path', default=os.path.expanduser('~/.local/bin'),
              help='The directory in which to look for the opm symlink')
def version(verbose, directory, path):
    """Print the version of the installed opm binary."""
    logger = get_logger(verbose)
    logger.debug(f'verbose: {verbose}')
    logger.debug(f'directory: {directory}')
    logger.debug(f'path: {path}')
    from osdk_manager.opm.update import opm_version
    click.echo(opm_version(directory=directory, path=path))



#!/usr/bin/env python3

import shlex
import subprocess
import os.path
from setuptools import setup, find_packages

base_dir = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(base_dir, "src", "osdk_manager",
                       "__about__.py"),
          encoding='utf-8') as f:
    exec(f.read(), about)

if os.path.isfile(os.path.join(base_dir, 'README.md')):
    with open(os.path.join(base_dir, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
else:
    long_description = '''
# {}

{}'''.format(about['__name__'], about['__summary__'])


def shell_run(command: str):
    return subprocess.check_output(shlex.split(command)).decode().strip()


def determine_version():
    os.chdir(base_dir)
    if os.path.isdir(os.path.join(base_dir, '..', '.git')):
        try:
            pkg_ver = shell_run('git describe --tags --abbrev=0')
            latest_commit = shell_run(
                'git rev-list HEAD --no-walk --max-count=1'
            )
            latest_tag_commit = shell_run(
                'git rev-list --tags --no-walk --max-count=1'
            )
            if latest_commit != latest_tag_commit:
                pkg_ver += '.dev{}'.format(shell_run(
                    'git rev-list {}..HEAD --count'.format(latest_tag_commit)
                ))
            tree_dirty = shell_run('git diff HEAD')
            if tree_dirty != '':
                pkg_ver += '-DIRTY'
            return pkg_ver
        except subprocess.CalledProcessError:
            return about.get('__version__') + about.get('__release__')
    else:
        return about.get('__version__') + about.get('__release__')


setup(
    name=about['__name__'],
    version=determine_version(),
    description=about['__summary__'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=about['__uri__'],
    author=about['__author__'],
    author_email=about['__email__'],
    license=about['__license__'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='kubernetes operator operator-sdk',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    python_requires='>=3.6',
    install_requires=about['__requires__'],
    entry_points={
        'console_scripts': [
            'osdk-manager=osdk_manager.cli:cli',
            'operator-sdk-manager=osdk_manager.cli:cli',
        ],
    },
)



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager about details.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains some basic variables used for setup.
"""


__title__ = "osdk-manager"
__name__ = __title__
__summary__ = "A script for managing Operator SDK-based operators."
__uri__ = "https://github.com/jharmison-redhat/osdk-manager"
__version__ = "0.3.0"
__release__ = "1"
__status__ = "Development"
__author__ = "James Harmison"
__email__ = "jharmison@gmail.com"
__license__ = "BSD-2-Clause"
__copyright__ = "2020 %s" % __author__
__requires__ = [
    'requests',
    'click',
    'lastversion',
    'python-gnupg',
    'PyYAML'
]



# SPDX-License-Identifier: BSD-2-Clause
"""osdk-manager command line utilities.

Manage osdk and opm binary installation, and help to scaffold, release, and
version Operator SDK-based Kubernetes operators.

This file contains multiple utility functions used by the cli modules
"""

import click


# We'll be using these repeatedly
def verbose_opt(func):
    """Wrap the function in a click.option for verbosity."""
    return click.option(
        "-v", "--verbose", count=True,
        help="Increase verbosity (specify multiple times for more)."
    )(func)
